
/**
 * Primary styles
 *
 * Author: IPython Development Team
 */

#main_app {
    width: 920px;
    margin: 30px auto 0px auto;
}

#tabs {
    border-style: none;
}

#tab1, #tab2 {
    padding: 1em 0em;
}

.list_toolbar {
    padding: 5px;
    height: 25px;
    line-height: 25px;
}

.toolbar_info {
    float: left;
}

.toolbar_buttons {
    float: right;
}

.list_header {
    height: 25px;
    line-height: 25px;
    padding: 3px 5px;
}



.list_item {
    height: 25px;
    line-height: 25px;
    padding: 3px 5px;
}

.notebook_item a {
    text-decoration: none;
}

.profile_col {
}

.status_col {
    float: right;
    width: 325px;
}

.engines_col {
    float: right;
    width: 325px;
}

.action_col {
    float: right;
}

.item_buttons {
    float: right;
}

.item_buttons .upload_button {
    color: darkred;
}

.highlight_text {
    color: blue;
}

.ui-tabs .ui-tabs-nav li a {
    padding: .3em .5em;
}
