{% extends page.html %}

{% block stylesheet %}

<link rel="stylesheet" href="{{static_url("css/login.css") }}" type="text/css"/>

{% end %}


{% block login_widget %}
{% end %}


{% block site %}

<div id="main_app">

    {% if login_available %}
    <form action="{{base_project_url}}login?next={{url_escape(next)}}" method="post">
        Password: <input type="password" class='ui-widget ui-widget-content' name="password" id="password_input">
        <input type="submit" value="Log in" id="login_submit">
    </form>
    {% end %}

    {% if message %}
      {% for key in message %}
        <div class="message {{key}}">
           {{message[key]}}
        </div>
      {% end %}
    {% end %}

<div/>

{% end %}


{% block script %}

<script src="{{static_url("js/loginmain.js") }}" type="text/javascript" charset="utf-8"></script>

{% end %}
